/***************************************************************************
 * functions.h 
 *
 *  Definiciones y cabeceras para el uso de funciones de análisis de tráfico 
 *  en las practicas de Arquitectura de Redes de Comunicaciones 2 de la EPS-UAM
 *
 *  Autor: Javier Ramos de Santiago
 *
 *  (C) 2012 EPS-UAM 
 ***************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <string.h>
#include <stdint.h>
#include <netinet/ip.h>
#include <linux/udp.h>
#include <linux/tcp.h>
#include <inttypes.h>
#define ETH_ALEN 6
#define ETH_TLEN 2
#define ETYPE_IP 0x0800UL
#define TCP 6
#define UDP 17
#define ICMP 1

#define OK 0
#define ERROR 1

#define NO_FILTER_IP 0xFFFFFFFFUL
#define NO_FILTER_PORT 0xFFFFU
#define NO_FILTER_ETHER 0xFFFFU
#define NO_FILTER_PROTO 0xFFU

#define MAX_PORTS 65536

typedef struct
{
	uint16_t port;
	uint32_t value;
}top_ports;
/*********************************************************************************************************************************************************
 * Función:createCDF
 * Descripción: Esta función crea un fichero que contiene los puntos necesarios para dibujar la ECDF dados los datos contenidos en el fichero filename_data.
 * Entrada:
 *
 *	-char *filename_data: nombre del fichero de entrada con los valores de las muestras
 *	-char *filename_cdf: nombre del fichero de salida con los valores para dibujar la ECDF
 *	
 * Salida:
 *	- 0 si no hay ningún problema
 *	- -1 si se produce algún error.
 *
 *  Autor: Jose Luis Garcia Dorado
 *
 *  (C) 2013 EPS-UAM 
 ****************************************************************************************************************************************************************/
int createCDF(char* filename_data, char* filename_cdf);
/*********************************************************************************************************************************************************
 * Función:getStats
 * Descripción: Esta función abre un fichero pcap y realiza el análisis de bytes,paquetes y duración en función de los filtros especificados como entrada.
 * Entrada:
 *	-uint32_t *bytes: Contador de bytes analizados. En caso de que haya filtros de 
 *	cualquier tipo (BPF, puertos, IPs) este valor debe almacenar el número de bytes 
 *	recibidos tras aplicar el filtro. Este valor debe rellenarse dentro de la función
 *	 
 *	 -uint32_t *packets: Contador de paquetes analizados. En caso de que haya filtros de 
 *	cualquier tipo (BPF, puertos, IPs) este valor debe almacenar el número de paquetes 
 *	recibidos tras aplicar el filtro. Este valor debe rellenarse dentro de la función
 *
 *	 -uint64_t *duration: Duración del tráfico analizado expresado en us. En caso de que haya filtros de 
 *	 cualquier tipo (BPF, puertos, IPs) este valor debe almacenar la duración de tráfico
 *	 tras aplicar el filtro. Este valor debe rellenarse dentro de la función
 *
 *	-uint16_t ethertype: Ethertype utilizado para filtrar. En caso de que se pase un valor diferente a 0xFFFF sólo deben
 *	analizarse aquellos paquetes que contengan el ethertype especificado.
 *	
 *	-uint32_t ip_src: Dirección IP origen utilizada para filtrar. En caso de que se pase un valor diferente a 0xFFFFFFFF sólo deben
 *	analizarse aquellos paquetes que contengan la ip origen especificada.
 *	
 *	-uint32_t ip_dst: Dirección IP destino utilizada para filtrar. En caso de que se pase un valor diferente a 0xFFFFFFFF sólo deben
 *	analizarse aquellos paquetes que contengan la ip destino especificada.
 *
 *	-uint16_t port_src: Puerto origen utilizado para filtrar (sólo aplica a paquetes TCP/UDP). En caso de que se pase un valor diferente a 0xFFFF sólo deben
 *	analizarse aquellos paquetes que contengan el puerto origen especificado.
 *	
 *	-uint16_t port_dst: Puerto destino utilizado para filtrar (sólo aplica a paquetes TCP/UDP). En caso de que se pase un valor diferente a 0xFFFF sólo deben
 *	analizarse aquellos paquetes que contengan el puerto destino especificado.
 *
 *	-uint8_t proto: Protocolo (a nivel IP) utilizado para filtrar. En caso de que se pase un valor diferente a 0xFF sólo deben
 *	analizarse aquellos paquetes que contengan el protocolo especificado.
 *
 *	-char *filter: En caso de que se pase un valor distinto de NULL debe aplicarse el filtro BPF contenido en filter al realizar el análisis del tráfico.
 *
 *	-char *filename: Nombre del fichero pcap a analizar
 *	
 * Salida:
 *	- 0 si no hay ningún problema
 *	- -1 si el nombre de fichero pasado es NULL o se produce algún error al procesar el fichero pcap especificado.
 *
 *  Autor: Javier Ramos de Santiago
 *
 *  (C) 2012 EPS-UAM 
 ****************************************************************************************************************************************************************/


int getStats(uint32_t *bytes,uint32_t *packets,uint64_t *duration,uint16_t ethertype,uint32_t ip_src, uint32_t ip_dst, uint16_t port_src, uint16_t port_dst, uint8_t proto, char * filter, char* filename);

/*********************************************************************************************************************************************************
 * Función:getTopNPort
 * Descripción: Esta función devuelve una lista de los N puertos (origen o destino) más populares agrupados por bytes o paquetes en función 
 *		de los parámetros pasados como argumento. 
 * Entrada:
 *
 *	-uint8_t direction: En caso de que la variable contenga el valor 0 se deben analizar los puertos origen. En caso de que contenga 1 se deben analizar los puertos destino.
 *	-uint8_t bytes_packets: En caso de que la variable contenga el valor 0 se debe contabilizar el número de bytes por puerto para realizar el Top. 
 *				 En caso de que contenga 1 se debe contabilizar el número de paquetes por puerto para realizar el Top. 
 *	-uint8_t N: Número de resultados a mostrar en el Top.
 *	-uint16_t *port: Array de tamaño N en el que deben guardarse los N puertos (origen o destino) más populares (por bytes o paquetes). Se pasa reservado.
 *	-uint16_t *value: Array de tamaño N en el que deben guardarse los valores (de bytes o paquetes) observados en los N puertos (origen o destino) más populares. Se pasa reservado
 *	-char *filter: En caso de que se pase un valor distinto de NULL debe aplicarse el filtro BPF contenido en filter al realizar el análisis de puertos.
 *
 *	-char *filename: Nombre del fichero pcap a analizar
 *	
 * Salida:
 *	- 0 si no hay ningún problema
 *	- -1 si el nombre de fichero pasado es NULL o se produce algún error al procesar el fichero pcap especificado.
 *
 *  Autor: Javier Ramos de Santiago
 *
 *  (C) 2012 EPS-UAM 
 ****************************************************************************************************************************************************************/
int getTopNPort(uint8_t direction,uint8_t bytes_packets,uint8_t N, uint16_t *port, uint32_t *value,u_char * filter, char* filename);

/*********************************************************************************************************************************************************
 * Función:getSizeECDFPort
 * Descripción: Esta función abre un fichero pcap y  escribe en el fichero apuntado por filename_out los valores correspondiente para la ECDF por tamaños 
 *filtrando por los puertos origen y destino que se pasan en port_src y port_dst.
 * Entrada:
 *	-char *filename: Nombre del fichero pcap a analizar
 *	-char *filename_out: Nombre del fichero de salida donde se volcaran los valores de la ECDF
 *	-uint16_t port_src: Puerto origen utilizado para filtrar (sólo aplica a paquetes TCP/UDP). En caso de que se pase un valor diferente a 0xFFFF sólo deben
 *	analizarse aquellos paquetes que contengan el puerto origen especificado.
 *	
 *	-uint16_t port_dst: Puerto destino utilizado para filtrar (sólo aplica a paquetes TCP/UDP). En caso de que se pase un valor diferente a 0xFFFF sólo deben
 *	analizarse aquellos paquetes que contengan el puerto destino especificado.
 *
 *	-char *filename: Nombre del fichero pcap a analizar
 *	
 * Salida:
 *	- 0 si no hay ningún problema
 *	- -1 si el nombre de fichero pasado es NULL o se produce algún error al procesar el fichero pcap especificado.
 *
 *  Autor: Javier Ramos de Santiago
 *
 *  (C) 2013 EPS-UAM 
 ****************************************************************************************************************************************************************/

int getSizeECDFPort(char* filename,char *filename_out,uint16_t port_src, uint16_t port_dst);

